import java.io.*;
import java.net.URL;
import javax.swing.*;

/**
 * Short demo class that can display the current day's Dilbert cartoon.
 * This is done by connecting to the known Dilbert web site (www.dilbert.com),
 * scanning the front page for the path of today's image file, and then
 * loading and displaying that image file.
 * 
 * The idea for this is not new. Similar programs circulate on various 
 * web sites and news groups - I am not sure where the idea originally 
 * came from.
 * 
 * @author (of this implementation) Michael Kolling
 * @version 1.0
 */

public class DilbertDisplayer {

    public static void main(String[] args) 
    {
        showDilbert();
    }
    
    /**
     * Make constructor private, since there is no use creating an instance.
     */
    private DilbertDisplayer()
    {
        // no need to ever create an instance...
    }
    
    /**
     * The main method of this class: find today's Dilbert cartoon and
     * show it on screen.
     */
    public static void showDilbert()
    {
        JFrame frame = new JFrame("DilbertDisplayer");
        JLabel label = new JLabel("    Connecting to Dilbert web site...    ");
        frame.getContentPane().add(label);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.show();

        URL url = getDilbertURL();
    
        if (url == null) {
            label.setText("  Could not find URL of today's comic. Giving up.    ");
        } 
        else {
            label.setText("    URL found. Loading...    ");
            ImageIcon icon = new ImageIcon(url);
            label.setIcon(icon);
            label.setText("");
            frame.pack();
        }
    }

    /**
     * Find out and construct the URL to the image file of today's Dilbert
     * cartoon.
     */
    private static URL getDilbertURL() 
    {
        try {
            // first, we connect to the dilbert web site
            URL dilberturl = new URL("http://www.dilbert.com");
            InputStream stream = dilberturl.openStream();
            BufferedReader reader = new BufferedReader(new InputStreamReader(stream));
        
            // now, we search for the line in the page that has the image source file name
            String line = reader.readLine();
            while (line != null) {
                int pathstart = line.indexOf("IMG SRC=\"/comics/dilbert/archive/images/dilbert");
                if (pathstart != -1) {
                    // okay, got it. construct a full URL to the image file
                    String path = line.substring(pathstart + 9);
                    path = path.substring(0, path.indexOf('"'));
                    return new URL("http://www.dilbert.com" + path);
                }
                line = reader.readLine();
            }
            return null;
        }
        catch(IOException exc) {
            return null;
        }
    }
}
